//---------------------------------------------------------------
// fileselect.c
//
// Very simple file selector for GP32
// 2003, Anders Granlund
//---------------------------------------------------------------
#include "gp32.h"
#include "ifilesel.h"	// file selector image

void fileselector_error(char* text);

#define FS_CPOS_OFFS_Y  28
#define FS_CPOS_OFFS_X	10
#define FS_MAXCPOS	16


GPDRAWSURFACE	screen[2];
int 		screenpage;
char		fileselector_fname[256];

//---------------------------------------------------------------
// Compare two strings
//---------------------------------------------------------------
int _islesser(char *str1, char *str2){
	int i;
	int len1,len2,len;
	int diff;
	// Get shortest string length
	len = len1 = gm_lstrlen(str1);
	len2 = gm_lstrlen(str2);
	if(len2<len) len = len2;
	for(i=0, diff=0; i<len && diff==0; i++){
		if( str1[i] < str2[i])
			return 1;
		if( str1[i] > str2[i])
			return 0;
	}
	// test if str1 is longer
	if(len1 > len2)
		return 0;
	// str2 is longer or equal
	return 1;
}

//---------------------------------------------------------------
// fileselector(char *path, int type)
//
//	path 		: start path
//
//	type		: 0 = files, 1 = directories
//
//	Note that this function will set the graphics to
//	8-bit mode.
//
//---------------------------------------------------------------
char* fileselector(char *path, int type){
	int		i,j;
	int		readcount;
	ERR_CODE	err;
	int		pkey;
	int		key;
	int		time;
	int		selected;
	char		*fname;
	int		cpos;
	int		cpos_max;
	int		dir_offs;
	GPFILEATTR	attr;
	GPDIRENTRY	*dir_entry_unmasked;
	GPDIRENTRY	*dir_entry;
	GPDIRENTRY	dir_temp;
	ulong		dir_count_unmasked;
	ulong		dir_count;
	char		filename[256];


	// Initialize file system
	GpFatInit();
	time = 0;
	dir_count = 0;
	dir_entry = NULL;
	dir_entry_unmasked = NULL;

	// Initialize video
	GpGraphicModeSet(8, NULL);
	GpLcdSurfaceGet( &screen[0], 0);
	GpLcdSurfaceGet( &screen[1], 1);
	GpSurfaceSet( &screen[0] );
	screenpage = 1;


	// Get directory information
	cpos	 = 0;
	cpos_max = FS_MAXCPOS;
	dir_offs = 0;

	// Set relative path
	GpRelativePathSet(path);

	// Enumerate files
	if( GpDirEnumNum("\\", &dir_count_unmasked) != SM_OK)
		fileselector_error("SMC Error. A");

	dir_entry_unmasked = gm_calloc( (int)dir_count_unmasked, sizeof(GPDIRENTRY));
	dir_entry 	   = gm_calloc( (int)dir_count_unmasked, sizeof(GPDIRENTRY));

	// Get the file list
	if( GpDirEnumList("\\", 0, dir_count_unmasked, dir_entry_unmasked, &readcount) != SM_OK)
		fileselector_error("SMC Error. B");

	for(i=0; i<readcount; i++){

		if((gm_compare( dir_entry_unmasked[i].name, ".") != 0) && (gm_compare( dir_entry_unmasked[i].name, "..") != 0) && (GpFileAttr(dir_entry_unmasked[i].name, &attr) == SM_OK)){

			//----------------------------------------------------
			// Directory
			//----------------------------------------------------
			if((attr.attr & 0x10) && (type==1)){
				dir_entry[dir_count] = dir_entry_unmasked[i];
				dir_count++;
			}

			//----------------------------------------------------
			// File
			//----------------------------------------------------
			else if(((attr.attr & 0x10)==0) && (type==0)){
				dir_entry[dir_count] = dir_entry_unmasked[i];
				dir_count++;
			}
		}
	}

	// make sure there are any files at all
	if( dir_count == 0)
		fileselector_error("No roms found.");

	// sort by filename
	if( dir_count > 1){
		for (i=0; i<dir_count-1; i++) {
  			for (j=0; j<dir_count-1-i; j++){
    				if( _islesser(dir_entry[j+1].name, dir_entry[j].name)){
    					gm_memcpy(&dir_temp, &dir_entry[j], sizeof(GPDIRENTRY));
    					gm_memcpy(&dir_entry[j], &dir_entry[j+1], sizeof(GPDIRENTRY));
    					gm_memcpy(&dir_entry[j+1], &dir_temp, sizeof(GPDIRENTRY));
  				}
			}
		}
	}


	// Set last cursor position
	if(dir_count<(FS_MAXCPOS+1))
		cpos_max = (dir_count-1);


	// wait for all buttons to be released
	GpKeyGetEx(&key); while(key!=0) GpKeyGetEx(&key);

        // Select a file please...
	selected = -1;
	while(selected==-1){

		// Handle input
		GpKeyGetEx(&key);
		if(key & GPC_VK_UP){
			if(cpos==0){
				if(dir_offs>0){
					dir_offs--;
				}
			}else{
				cpos--;
			}
		}
		else if(key & GPC_VK_DOWN){
			if(cpos>=FS_MAXCPOS){
				if((cpos+dir_offs+1)<dir_count){
					dir_offs++;
				}
			}else if (cpos<cpos_max){
				cpos++;
			}
		}
		else if(key & GPC_VK_FA){
			selected = (dir_offs + cpos);
		}



		// Render background image
        	gm_memcpy(screen[screenpage].ptbuffer, ifilesel, 320*240);

		// Render cursor
		GpRectFill(NULL, &screen[screenpage], 10, FS_CPOS_OFFS_Y + (cpos*12), 300, 13, 63);

		// Render directory entries
		for(i=0; i<=cpos_max; i++){

			GpTextOut(NULL, &screen[screenpage], 16, FS_CPOS_OFFS_Y + (i*12) , dir_entry[i+dir_offs].name, 0x00);

		}

        	// swap buffers
		GpSurfaceSet( &screen[screenpage] );
		screenpage = 1 - screenpage;

		// wait a little while
		while( (GpTickCountGet() - time) < 100 ) {}
		time = GpTickCountGet();
	}

	strcpy( fileselector_fname, dir_entry[selected].name);

	if(dir_entry!=NULL)
		gm_free(dir_entry);
	if(dir_entry_unmasked!=NULL)
		gm_free(dir_entry_unmasked);

	return fileselector_fname;
}


//---------------------------------------------------------------
//
// fileselector_error(char *text)
//
//	Display an error message and wait forever
//
//---------------------------------------------------------------
void fileselector_error(char* text){
	int x,w;

	// Render background image
       	gm_memcpy(screen[screenpage].ptbuffer, ifilesel, 320*240);

	// Print message
	w = GpTextWidthGet(text);
	x = (320 - w) >> 1;
	GpTextOut(NULL, &screen[screenpage], x, 120 , text, 0x00);

       	// swap buffers
	GpSurfaceSet( &screen[screenpage] );
	screenpage = 1 - screenpage;

	// wait forever...
	while(1){
		// haha! you're stuck here aren't ya!
	}

}

